/* ============================================================================
 *                          p i n _ i s r _ h a n d l e r . c
 * ============================================================================
 * Cytonome CAB 3.0 Cypress CY8C5868AXI-LP035 Firmware
 * Copyright Cytonome ST, LLC 2022
 * All Rights Reserved
 *
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF Cytonome ST, LLC.
 
 * Revision History:
 *  001 10 Jul 22 <sek> created
 *  002 25 Jul 22 <sek> comments
 *  003 25 Aug 22 <sek> RuntimeError() implementation
 *  003 26 Aug 22 <sek> disable RuntimeError(PIN_ISR_ERROR_KEEPALIVE_FLAG_OVERRUN) (flooding the error buffer. Why?)
 *  004 20 Jan 23 <sek> moved overrun error checks to Start_LED_Strobe_In_Check(), Start_Keepalive_In_Check()
 *  005 20 Jan 23 <sek> PIN_ISR_ERROR_KEEPALIVE_NO_INTERRUPT, PIN_ISR_ERROR_LED_STROBE_NO_INTERRUPT, fgKeepalive_Interrupt_Flag
 *  006 20 Jan 23 <sek> limit LED_Strobe_In interrupts to just one per setup
 *
 * ============================================================================
*/

#include <gsfstd.h>

#include <main.h>
#include "debug.h"

#include "pin_isr_handler.h"
#include "eeprom_interface.h"

enum tagIOControlErrors
{
	PIN_ISR_ERROR_NONE = PIN_ISR_ERROR_BASE,
	PIN_ISR_ERROR_UNEXPECTED_TICK,		// 1 unexpected timer tick event
	PIN_ISR_ERROR_UNEXPECTED_EVENT,		// 2 unexpected event
	PIN_ISR_ERROR_INVALID_STATE,		// 3 not a valid state
	PIN_ISR_ERROR_INVALID_SUBSTATE,		// 4 not a valid state

	PIN_ISR_ERROR_KEEPALIVE_FLAG_OVERRUN,	// 5 activeFlag overrun
	PIN_ISR_ERROR_KEEPALIVE_NO_INTERRUPT,	// 6 no interrupt has occured
    
	PIN_ISR_ERROR_LED_STROBE_FLAG_OVERRUN,	// 7 activeFlag overrun
	PIN_ISR_ERROR_LED_STROBE_NO_INTERRUPT,	// 8 no interrupt has occured
    
	PIN_ISR_ERROR_UNPROCESSED_EVENT = PIN_ISR_ERROR_BASE + 0x0F

};

BOOL  fgStrobe_In_activeFlag = FALSE;
BOOL  fgKeepalive_In_activeFlag = FALSE;
BOOL  fgprevHV_Interlock_State = FALSE;
BOOL  fgKeepalive_Interrupt_Flag = FALSE;
BOOL  fgStrobe_In_Interrupt_Flag = FALSE;

//============================================================================
//
CY_ISR(LED_Strobe_In_Handler) 
{
    fgStrobe_In_activeFlag = TRUE;
    fgStrobe_In_Interrupt_Flag = TRUE;
    
    // Clear the pending interrupt
    //
    LED_Strobe_In_ClearInterrupt();

    LED_Strobe_In_ISR_Disable();             // disable interrupts until next call to Start_LED_Strobe_In_Check()
}

// start wait for interrupt
void Start_LED_Strobe_In_Check(void)
{
    // make certain that the flag has been read since the last call to Start_LED_Strobe_In_Check()
    if (fgStrobe_In_activeFlag IS_TRUE)
    {
        RuntimeError(PIN_ISR_ERROR_LED_STROBE_FLAG_OVERRUN);   // flag error
    }
    
    fgStrobe_In_activeFlag = FALSE;
    fgStrobe_In_Interrupt_Flag = FALSE;

    LED_Strobe_In_ISR_Enable();             // enable interrupts (for just ONE interrupt)
}

// get most recent state; indicates activity
BOOL GetLEDStrobeIn_ActiveFlag(void)
{
    // make certain that an interrupt has occured
    if (fgStrobe_In_Interrupt_Flag IS_FALSE)
    {
       RuntimeError(PIN_ISR_ERROR_LED_STROBE_NO_INTERRUPT);   // flag error
    }
    
    BOOL retVal = fgStrobe_In_activeFlag;
    fgStrobe_In_activeFlag = FALSE;
    
    return(retVal);
 
}


void LED_Strobe_In_initialize(void)
{
    LED_Strobe_In_ISR_StartEx(LED_Strobe_In_Handler);
    fgStrobe_In_activeFlag = FALSE;
}

//============================================================================
//

// From Emanuel Machado:
// The Keepalive is not a static line. It is a square wave that pulses at around 12.2 kHz (or 24.4 kHz, I’m not too sure). 
// This way you know that it is something “intelligent” that is driving it.

// NOTE: the Keepalive edge rate is too fast for us to respond to every one.

CY_ISR(Keepalive_In_Handler) 
{
    fgKeepalive_In_activeFlag = TRUE;
    fgKeepalive_Interrupt_Flag = TRUE;

    // Clear the pending interrupt
    //
    Keepalive_In_ClearInterrupt();
    //Keepalive_In_ISR_ClearPending();
    
    Keepalive_In_ISR_Disable();             // disable interrupts until next call to Start_Keepalive_In_Check()
}

// start wait for interrupt
void Start_Keepalive_In_Check(void)
{
    // make certain that the flag has been read since the last call to Start_Keepalive_In_Check()
    if (fgKeepalive_In_activeFlag IS_TRUE)
    {
       RuntimeError(PIN_ISR_ERROR_KEEPALIVE_FLAG_OVERRUN);   // flag error
    }
    
    fgKeepalive_In_activeFlag = FALSE;
    fgKeepalive_Interrupt_Flag = FALSE;     // initialize to no interrupt has occured
    
    Keepalive_In_ISR_Enable();             // enable interrupts (for just ONE interrupt)
}

// get most recent state; indicates activity
BOOL GetKeepaliveIn_ActiveFlag(void)
{
    // make certain that an interrupt has occured
    if (fgKeepalive_Interrupt_Flag IS_FALSE)
    {
       RuntimeError(PIN_ISR_ERROR_KEEPALIVE_NO_INTERRUPT);   // flag error
    }

    BOOL retVal = fgKeepalive_In_activeFlag;
    fgKeepalive_In_activeFlag = FALSE;              // clear the flag to that it can be set by another interrupt
    
    return(retVal);
    
}

void Keepalive_In_initialize(void)
{
    Keepalive_In_ISR_StartEx(Keepalive_In_Handler);
    fgKeepalive_In_activeFlag = FALSE;
    fgKeepalive_Interrupt_Flag = FALSE;
}

//============================================================================
//

// called at 1Hz from led_interface.c: leds_update(void)

BOOL GetHV_Interlock_State(void)
{
    BOOL new_state = (BOOL)HV_Interlock_State_Read();
    if (new_state IS fgprevHV_Interlock_State)
    {
        // no change, so no event
        return new_state;
    }
    else
    {
        // state has changed
        EEPROM_inc_interlock_events_count();
        fgprevHV_Interlock_State = new_state;
    }
    
    return new_state;
    
}

/* [] END OF FILE */
